#include "Scanner.h"
#include <sstream>
#include <cctype>

int Scanner::process_const_table(int index) {
    return 0;
}

int Scanner::process_identifier_table(int index) {
    std::stringstream buffer;
    int old_index = index;
    if (std::isalpha(m_source_code[index])) {
        buffer << m_source_code[index];
        index += 1;
        while(std::isalnum(m_source_code[index])) {
            buffer << m_source_code[index];
            index += 1;
        }

        std::string identifier = buffer.str();
        for (const auto& key : m_tables.KeyTable) {
            if (identifier == key.second) {
                return 0;
            }
        }

        m_tables.IdTable.insert({identifier_index, identifier});
        m_token_list.push_back(Token{identifier_index, ID_TABLE});
        identifier_index++;
        return index - old_index;
    } else {
        return 0;
    }

}

int Scanner::process_key_table(int index) {
    int max_len = 0;
    int found_key = -1; // 存储找到的关键字编号

    // 遍历关键字表
    for (const auto& pair : m_tables.KeyTable) {
        const std::string& keyword = pair.second;
        int len = keyword.length();

        // 检查剩余长度是否足够
        if (index + len > m_source_code.length()) {
            continue;
        }

        // 比较子串是否匹配关键字
        if (m_source_code.substr(index, len) == keyword) {
            // 检查关键字后是否紧跟字母/数字/下划线
            if (index + len < m_source_code.length()) {
                char next_char = m_source_code[index + len];
                if (isalnum(next_char) || next_char == '_') {
                    continue; // 是标识符的一部分，跳过
                }
            }

            // 更新最长匹配（解决"float32"和"float64"的冲突）
            if (len > max_len) {
                max_len = len;
                found_key = pair.first;
            }
        }
    }

    // 找到有效关键字
    if (max_len > 0) {
        Token token({ found_key, KEY_TABLE });
        m_token_list.push_back(token);
        return max_len;
    }

    return 0; // 未识别到关键字
}

int Scanner::process_punct_table(int index) {
    //identify the Punct in map
    string s;
    int n = this->m_source_code.size();
    char c1= this->m_source_code[index];
    char c2 = '@';
    char c3 = '@';
    if (index + 1 < n) c2 = this->m_source_code[index + 1];
    if (index + 2 < n) c3 = this->m_source_code[index + 2];
    if (((c1 == c2 && c2 == '<') || (c1 == c2&& c2 == '>'))&& c3 =='=') {
        this->m_token_list.push_back({c1=='<'?31:32,PUNCT_TABLE});
        return 3;
    }
    if (c1 == c2 ) {
        if (c1 =='=') {
            this->m_token_list.push_back({15,PUNCT_TABLE});
            return 2;
        }
        if (c1 =='|') {
            this->m_token_list.push_back({21,PUNCT_TABLE});
            return 2;
        }
        if (c1=='&') {
            this->m_token_list.push_back({20,PUNCT_TABLE});
            return 2;
        }
        if (c1=='<') {
            this->m_token_list.push_back({9,PUNCT_TABLE});
            return 2;
        }
        if (c1=='>') {
            this->m_token_list.push_back({10,PUNCT_TABLE});
            return 2;
        }
    }

    string t;
    t.push_back(c1);
    t.push_back(c2);
    for (auto e : this->m_tables.PunctTable) {
        if (e.second == t) {
            this->m_token_list.push_back({e.first,PUNCT_TABLE});
            return 2;
        }
    }

    t.pop_back();
    for (auto e : this->m_tables.PunctTable) {
        if (e.second == t) {
            this->m_token_list.push_back({e.first,PUNCT_TABLE});
            return 1;
        }
    }
    return 0;
}
